

///////////////////////////////////////////////
// Gestion des volets Velux avec télécommandes 433 Mhz protocole EV1527
// Arduino Nano
// 08/2022  alex.pruvost@laposte.net
///////////////////////////////////////////////

// Données sur les télécommandes utilisées :
// 2 boutons architecture chip EV1527
// Code capté avec librairie RF433any :
//
//  Durations are in microseconds
//  Data is output in hexdecimal
//  When high durations are equal to low, they are output as zero
//  Codes: (I)nconsistent, (S)ync, (U)nknown, (T)ribit, tribit i(N)verted, (M)anchester
//  I:  Initseq = 'high' radio signal sent once
//  LS: Low Short duration
//  LL: Low Long duration
//  HS: High Short duration
//  HL: High Long duration
//  S: Separator (typically separating repeats)
//  U: Low signal prefix (zero most frequently)
//  V: High signal prefix (zero most frequently)
//  Y: First low non-coding signal (non-zero only if encoding is tribit inverted)
//  Z: Last low signal
//
// Télécommande 1 ON :
// Decoded: yes, err: 0, code: T, rep: 1, bits: 24, data: bd e6 32
//         I=0, LS=350, LL=1056, HS=0, HL=0, S=10868, U=0, V=0, Y=0, Z=340
// Télécommande 1 OFF :
// Decoded: yes, err: 0, code: T, rep: 1, bits: 24, data: bd e6 34
//         I=0, LS=352, LL=1056, HS=0, HL=0, S=10876, U=0, V=0, Y=0, Z=348
//
// Télécommande 2 ON :
// Decoded: yes, err: 0, code: T, rep: 1, bits: 24, data: fe 9c 52
//         I=0, LS=352, LL=1054, HS=0, HL=0, S=10804, U=0, V=0, Y=0, Z=344
// Télécommande 2 OFF :
// Decoded: yes, err: 0, code: T, rep: 1, bits: 24, data: fe 9c 54
//         I=0, LS=346, LL=1054, HS=0, HL=0, S=10804, U=0, V=0, Y=0, Z=344
//
// Télécommande 3 ON :
// Decoded: yes, err: 0, code: T, rep: 1, bits: 24, data: 1b a8 02
//         I=0, LS=350, LL=1052, HS=0, HL=0, S=10788, U=0, V=0, Y=0, Z=360
// Télécommande 3 OFF :
// Decoded: yes, err: 0, code: T, rep: 1, bits: 24, data: 1b a8 04
//         I=0, LS=352, LL=1052, HS=0, HL=0, S=10796, U=0, V=0, Y=0, Z=360

// Données sur le récepteur utilisé :
// modèle HFY-J69B
//

#include <RF433recv.h>
#include <Arduino.h>
#include <avr/wdt.h>

// ---------------------------------
// Constantes de calibrage du système :

const long TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION = 40000; // en millisecondes

// ---------------------------------
// Assignation des entrées/sorties :

const byte PIN_RFINPUT = 2;
const byte INT_RFINPUT = 0;

const byte ledPin = 13;

const byte VOLET1_SENSMONTEE = 5;
const byte VOLET1_SENSDESCENTE = 6;

const byte VOLET2_SENSMONTEE = 7;
const byte VOLET2_SENSDESCENTE = 8;

const byte VOLET3_SENSMONTEE = 9;
const byte VOLET3_SENSDESCENTE = 10;

const byte VOLET4_SENSMONTEE = 11;
const byte VOLET4_SENSDESCENTE = 12;

// ---------------------------------
// Variables globales :

unsigned long timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_1_AlaFinDeSaManoeuvre;
unsigned long timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_2_AlaFinDeSaManoeuvre;
unsigned long timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_3_AlaFinDeSaManoeuvre;
unsigned long timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_4_AlaFinDeSaManoeuvre;

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// Fonctions et méthodes

void callback_anycode(const BitVector *recorded)
{
    Serial.print(F("Code recu lors de l'appui du bouton : "));
    char *printed_code = recorded->to_str();

    if (printed_code)
    {
        Serial.print(recorded->get_nb_bits());
        Serial.print(F(" bits: ["));
        Serial.print(printed_code);
        Serial.print(F("]\n"));

        free(printed_code);
    }
}

void appuiTelecommande1_ON(const BitVector *recorded)
{ // sens montée du volet 1 - Télécommande mobile chambre d'amis
    Serial.print(F("appuiTelecommande1_ON\n"));

    if (digitalRead(VOLET1_SENSMONTEE) == LOW && digitalRead(VOLET1_SENSDESCENTE) == LOW)
    {                                          // si le volet est à l'arrêt
        digitalWrite(VOLET1_SENSMONTEE, HIGH); // dans ce cas le faire monter
        Serial.print(F("Volet 1 en train de monter\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_1_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET1_SENSMONTEE, LOW);
        digitalWrite(VOLET1_SENSDESCENTE, LOW);
        Serial.print(F("Volet 1 arrêté\n"));
    }
}

void appuiTelecommande1_OFF(const BitVector *recorded)
{ // sens descente du volet 1 - Télécommande mobile chambre d'amis
    Serial.print(F("appuiTelecommande1_OFF\n"));

    if (digitalRead(VOLET1_SENSMONTEE) == LOW && digitalRead(VOLET1_SENSDESCENTE) == LOW)
    {                                            // si le volet est à l'arrêt
        digitalWrite(VOLET1_SENSDESCENTE, HIGH); // dans ce cas le faire descendre
        Serial.print(F("Volet 1 en train de descendre\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_1_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET1_SENSMONTEE, LOW);
        digitalWrite(VOLET1_SENSDESCENTE, LOW);
        Serial.print(F("Volet 1 arrêté\n"));
    }
}

void appuiTelecommande4_MONTEE(const BitVector *recorded)
{ // sens montée du volet 1 - Télécommande murale chambre d'amis
    Serial.print(F("appuiTelecommande4_MONTEE\n"));

    if (digitalRead(VOLET1_SENSMONTEE) == LOW && digitalRead(VOLET1_SENSDESCENTE) == LOW)
    {                                          // si le volet est à l'arrêt
        digitalWrite(VOLET1_SENSMONTEE, HIGH); // dans ce cas le faire monter
        Serial.print(F("Volet 1 en train de monter\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_1_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET1_SENSMONTEE, LOW);
        digitalWrite(VOLET1_SENSDESCENTE, LOW);
        Serial.print(F("Volet 1 arrêté\n"));
    }
}

void appuiTelecommande4_DESCENTE(const BitVector *recorded)
{ // sens descente du volet 1 - Télécommande murale chambre d'amis
    Serial.print(F("appuiTelecommande4_DESCENTE\n"));

    if (digitalRead(VOLET1_SENSMONTEE) == LOW && digitalRead(VOLET1_SENSDESCENTE) == LOW)
    {                                            // si le volet est à l'arrêt
        digitalWrite(VOLET1_SENSDESCENTE, HIGH); // dans ce cas le faire descendre
        Serial.print(F("Volet 1 en train de descendre\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_1_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET1_SENSMONTEE, LOW);
        digitalWrite(VOLET1_SENSDESCENTE, LOW);
        Serial.print(F("Volet 1 arrêté\n"));
    }
}

void appuiTelecommande2_ON(const BitVector *recorded)
{ // sens montée du volet 2 - Télécommande mobile palier
    Serial.print(F("appuiTelecommande2_ON\n"));

    if (digitalRead(VOLET2_SENSMONTEE) == LOW && digitalRead(VOLET2_SENSDESCENTE) == LOW)
    {                                          // si le volet est à l'arrêt
        digitalWrite(VOLET2_SENSMONTEE, HIGH); // dans ce cas le faire monter
        Serial.print(F("Volet 2 en train de monter\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_2_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET2_SENSMONTEE, LOW);
        digitalWrite(VOLET2_SENSDESCENTE, LOW);
        Serial.print(F("Volet 2 arrêté\n"));
    }
}

void appuiTelecommande2_OFF(const BitVector *recorded)
{ // sens descente du volet 2 - Télécommande mobile palier
    Serial.print(F("appuiTelecommande2_OFF\n"));

    if (digitalRead(VOLET2_SENSMONTEE) == LOW && digitalRead(VOLET2_SENSDESCENTE) == LOW)
    {                                            // si le volet est à l'arrêt
        digitalWrite(VOLET2_SENSDESCENTE, HIGH); // dans ce cas le faire descendre
        Serial.print(F("Volet 2 en train de descendre\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_2_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET2_SENSMONTEE, LOW);
        digitalWrite(VOLET2_SENSDESCENTE, LOW);
        Serial.print(F("Volet 2 arrêté\n"));
    }
}

void appuiTelecommande5_MONTEE(const BitVector *recorded)
{ // sens montée du volet 2 - Télécommande murale palier
    Serial.print(F("appuiTelecommande5_MONTEE\n"));

    if (digitalRead(VOLET2_SENSMONTEE) == LOW && digitalRead(VOLET2_SENSDESCENTE) == LOW)
    {                                          // si le volet est à l'arrêt
        digitalWrite(VOLET2_SENSMONTEE, HIGH); // dans ce cas le faire monter
        Serial.print(F("Volet 2 en train de monter\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_2_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET2_SENSMONTEE, LOW);
        digitalWrite(VOLET2_SENSDESCENTE, LOW);
        Serial.print(F("Volet 2 arrêté\n"));
    }
}

void appuiTelecommande5_DESCENTE(const BitVector *recorded)
{ // sens descente du volet 2 - Télécommande murale palier
    Serial.print(F("appuiTelecommande5_DESCENTE\n"));

    if (digitalRead(VOLET2_SENSMONTEE) == LOW && digitalRead(VOLET2_SENSDESCENTE) == LOW)
    {                                            // si le volet est à l'arrêt
        digitalWrite(VOLET2_SENSDESCENTE, HIGH); // dans ce cas le faire descendre
        Serial.print(F("Volet 2 en train de descendre\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_2_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET2_SENSMONTEE, LOW);
        digitalWrite(VOLET2_SENSDESCENTE, LOW);
        Serial.print(F("Volet 2 arrêté\n"));
    }
}

void appuiTelecommande3_ON(const BitVector *recorded)
{ // sens montée du volet 4 - Télécommande mobile chambre agathe coté rue
    Serial.print(F("appuiTelecommande3_ON\n"));

    if (digitalRead(VOLET4_SENSMONTEE) == LOW && digitalRead(VOLET4_SENSDESCENTE) == LOW)
    {                                          // si le volet est à l'arrêt
        digitalWrite(VOLET4_SENSMONTEE, HIGH); // dans ce cas le faire monter
        Serial.print(F("Volet 4 en train de monter\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_4_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET4_SENSMONTEE, LOW);
        digitalWrite(VOLET4_SENSDESCENTE, LOW);
        Serial.print(F("Volet 4 arrêté\n"));
    }
}

void appuiTelecommande3_OFF(const BitVector *recorded)
{ // sens descente du volet 4 - Télécommande mobile chambre agathe coté rue
    Serial.print(F("appuiTelecommande3_OFF\n"));

    if (digitalRead(VOLET4_SENSMONTEE) == LOW && digitalRead(VOLET4_SENSDESCENTE) == LOW)
    {                                            // si le volet est à l'arrêt
        digitalWrite(VOLET4_SENSDESCENTE, HIGH); // dans ce cas le faire descendre
        Serial.print(F("Volet 4 en train de descendre\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_4_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET4_SENSMONTEE, LOW);
        digitalWrite(VOLET4_SENSDESCENTE, LOW);
        Serial.print(F("Volet 4 arrêté\n"));
    }
}

void appuiTelecommande6_MONTEE(const BitVector *recorded)
{ // sens montée du volet 4 - Télécommande murale chambre agathe coté rue
    Serial.print(F("appuiTelecommande6_MONTEE\n"));

    if (digitalRead(VOLET4_SENSMONTEE) == LOW && digitalRead(VOLET4_SENSDESCENTE) == LOW)
    {                                          // si le volet est à l'arrêt
        digitalWrite(VOLET4_SENSMONTEE, HIGH); // dans ce cas le faire monter
        Serial.print(F("Volet 4 en train de monter\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_4_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET4_SENSMONTEE, LOW);
        digitalWrite(VOLET4_SENSDESCENTE, LOW);
        Serial.print(F("Volet 4 arrêté\n"));
    }
}

void appuiTelecommande6_DESCENTE(const BitVector *recorded)
{ // sens descente du volet 4 - Télécommande murale chambre agathe coté rue
    Serial.print(F("appuiTelecommande6_DESCENTE\n"));

    if (digitalRead(VOLET4_SENSMONTEE) == LOW && digitalRead(VOLET4_SENSDESCENTE) == LOW)
    {                                            // si le volet est à l'arrêt
        digitalWrite(VOLET4_SENSDESCENTE, HIGH); // dans ce cas le faire descendre
        Serial.print(F("Volet 4 en train de descendre\n"));
        timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_4_AlaFinDeSaManoeuvre = millis() + TEMPSDEMANOEUVREMAXIDUVOLETAVANTMISEHORSTENSION;
    }
    else
    {
        // sinon s'il était en train de monter ou descendre, arrêter tout
        digitalWrite(VOLET4_SENSMONTEE, LOW);
        digitalWrite(VOLET4_SENSDESCENTE, LOW);
        Serial.print(F("Volet 4 arrêté\n"));
    }
}

RF_manager rf(PIN_RFINPUT, INT_RFINPUT);

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
void setup()
{
    // put your setup code here, to run once:

    wdt_disable(); // disable wdt at the first of code to prevent it always reboot

    pinMode(ledPin, OUTPUT);

    pinMode(VOLET1_SENSMONTEE, OUTPUT);
    pinMode(VOLET1_SENSDESCENTE, OUTPUT);

    pinMode(VOLET2_SENSMONTEE, OUTPUT);
    pinMode(VOLET2_SENSDESCENTE, OUTPUT);

    pinMode(VOLET3_SENSMONTEE, OUTPUT);
    pinMode(VOLET3_SENSDESCENTE, OUTPUT);

    pinMode(VOLET4_SENSMONTEE, OUTPUT);
    pinMode(VOLET4_SENSDESCENTE, OUTPUT);

    pinMode(PIN_RFINPUT, INPUT);

    // open the serial port
    Serial.begin(115200);

    // Replace the below code with your telecommand timings.
    // See RF433any (https://github.com/sebmillet/RF433any) as to how to
    // work out these timings. In particular, you can use example:
    //   examples/01_main/01_main.ino
    // to output exactly what is needed to call register_Receiver().
    // This example complete URL is:
    //   https://github.com/sebmillet/RF433any/blob/main/examples/01_main/01_main.ino

    // Télécommande chip EV1527
    rf.register_Receiver(
        RFMOD_TRIBIT,     // mod
        10820,            // initseq
        0,                // lo_prefix
        0,                // hi_prefix
        0,                // first_lo_ign
        350,              // lo_short
        1054,             // lo_long
        0,                // hi_short (0 => take lo_short)
        0,                // hi_long  (0 => take lo_long)
        352,              // lo_last
        10820,            // sep
        24,               // nb_bits
        callback_anycode, // callback (optional ; just to show received code) - Sert aussi à récupérer les codes des nouvelles télécommandes
        0                 // delay between two calls to callback
    );

    // Enrolement télécommande 1 mobile (chambre amis)  :
    rf.register_callback(appuiTelecommande1_ON, 1000,
                         new BitVector(24, 3, 0xbd, 0xe6, 0x32));
    rf.register_callback(appuiTelecommande1_OFF, 1000,
                         new BitVector(24, 3, 0xbd, 0xe6, 0x34));

    // Enrolement télécommande 2 mobile (palier):
    rf.register_callback(appuiTelecommande2_ON, 1000,
                         new BitVector(24, 3, 0xfe, 0x9c, 0x52));
    rf.register_callback(appuiTelecommande2_OFF, 1000,
                         new BitVector(24, 3, 0xfe, 0x9c, 0x54));

    // Enrolement télécommande 3 mobile (chambre Agathe):
    rf.register_callback(appuiTelecommande3_ON, 1000,
                         new BitVector(24, 3, 0x1b, 0xa8, 0x02));
    rf.register_callback(appuiTelecommande3_OFF, 1000,
                         new BitVector(24, 3, 0x1b, 0xa8, 0x04));

    // Enrolement télécommande 4 murale (chambre amis)  :
    rf.register_callback(appuiTelecommande4_MONTEE, 1000,
                         new BitVector(24, 3, 0x90, 0xb3, 0x64));
    rf.register_callback(appuiTelecommande4_DESCENTE, 1000,
                         new BitVector(24, 3, 0x90, 0xb3, 0x62));

    // Enrolement télécommande 5 murale (palier)  :
    rf.register_callback(appuiTelecommande5_MONTEE, 1000,
                         new BitVector(24, 3, 0x90, 0x3e, 0x54));
    rf.register_callback(appuiTelecommande5_DESCENTE, 1000,
                         new BitVector(24, 3, 0x90, 0x3e, 0x52));

    // Enrolement télécommande 6 murale (chambre Agathe)  :
    rf.register_callback(appuiTelecommande6_MONTEE, 1000,
                         new BitVector(24, 3, 0x91, 0x16, 0x74));
    rf.register_callback(appuiTelecommande6_DESCENTE, 1000,
                         new BitVector(24, 3, 0x91, 0x16, 0x72));

    rf.activate_interrupts_handler();

    wdt_enable(WDTO_4S); // Démarrer le watchdog avec une periode de reset de 4 secondes si le programme ne répond plus

    Serial.print(F("Démarrage de la carte\n"));

} // end setup

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// Vecteurs d'interruption

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
void loop()
{

    // Autoresetting the microcontroller every day
    // if ( millis()  >= 1000) resetFunc(); //call reset every 1 second.
    // if (millis() >= 60000) resetFunc(); // call reset every 60 seconds (1 Minute).
    // if ( millis()  >= 3600000) resetFunc(); //call reset every 60 mins (1 Hour)
    if (millis() >= 86400000)
    { // call reset every 24 hours (1 Day). .
        while (1)
        {// blocks untils the WDT triggers
        }
    }
    // if ( millis()  >= 604800000 ) resetFunc(); //call reset every 7 days (1 Week).
    // if ( millis()  >= 2592000000) resetFunc(); //call reset every 30 days (1 M

    wdt_reset();

    rf.do_events(); // lance l'écoute du récepteur radio

    if (millis() == timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_1_AlaFinDeSaManoeuvre)
    { // Mettre hors tension le volet au bout d'un certain temps qu'il n'a plus eu de commande
        digitalWrite(VOLET1_SENSMONTEE, LOW);
        digitalWrite(VOLET1_SENSDESCENTE, LOW);
        Serial.print(F("Volet 1 mis hors tension suite idle timeout\n"));
    }

    if (millis() == timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_2_AlaFinDeSaManoeuvre)
    { // Mettre hors tension le volet au bout d'un certain temps qu'il n'a plus eu de commande
        digitalWrite(VOLET2_SENSMONTEE, LOW);
        digitalWrite(VOLET2_SENSDESCENTE, LOW);
        Serial.print(F("Volet 2 mis hors tension suite idle timeout\n"));
    }

    if (millis() == timeStampDeQuandIlFaudraMettreHorsTensionLeVolet_4_AlaFinDeSaManoeuvre)
    { // Mettre hors tension le volet au bout d'un certain temps qu'il n'a plus eu de commande
        digitalWrite(VOLET4_SENSMONTEE, LOW);
        digitalWrite(VOLET4_SENSDESCENTE, LOW);
        Serial.print(F("Volet 4 mis hors tension suite idle timeout\n"));
    }
}

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// Fonctions et méthodes
